/*  MaViLla: Marcador y Visualizador de Llamadas
 *
 *  Copyright (C) 2011  Area de Servicios Telematicos
 *			CSIRC
 *			Universidad de Granada
 *			telematica@ugr.es
 *  
 *  Version: 1.1
 *  Last Modified: 30 May 2011
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef PRINCIPAL_H
#define PRINCIPAL_H

#include <QMainWindow>
#include <QSystemTrayIcon>
#include <QtNetwork/QtNetwork>
#include <QUrl>
#include "registro_llamadas.h"
#include "imagen.h"
#include "entrada.h"
#include "edicion.h"
#include "errores.h"
#include "llamada_saliente.h"
#include "main.h"

QT_BEGIN_NAMESPACE

class QUdpSocket;
class QNetworkReply;
class QMenu;
class QCloseEvent;
class QLabel;

QT_END_NAMESPACE

namespace Ui {
    class Principal;
}


class Principal : public QMainWindow
{
    Q_OBJECT

protected:

    /**
    @brief Personaliza el metodo closeEvent
    @param event Informacion acerca del evento

    Sobrescribimos el metodo para que cuando se produzca el evento de cerrar, la aplicacion
    se minimice.
    */
    void closeEvent(QCloseEvent *event);

private slots:

    /**
    @brief Slot que se ejecutara cuando pulsemos enter dentro del cuadro buscar

    Cuando escribimos algo dentro del cuadro de texto 'buscar' y pulsamos enter se
    ejecuta este metodo que nos permite filtrar los contactos mostrando solo aquellos
    que contengan alguna conincidencia con el patron buscado.
    */
    void on_lineEdit_returnPressed();

    /**
    @brief Slot que se ejecutara cuando hagamos click sobre el checkbox 'Mostrar solo contactos'

    Se encarga de activar la opcion de filtrado de llamadas para que el programa solo nos muestre las llamadas
    que provengan de alguno de nuestros contactos.
    */
    void on_checkBox_stateChanged(int );

    /**
    @brief Slot que se ejecutara cuando recibamos algun mensaje en el puerto esecificado

    Procesa la cadena de entrada que recibimos a traves del socket, obteniendo el nombre y
    el telefono de la persona que llama mostrandolo en un dialogo.
    */
    void procesarEntrada();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'salir'

    Cierra la aplicacion informandonos de si hemos realizado algun cambio que no hemos guardado
    y queremos hacerlo.
    */
    void cerrar();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'llamar' oclickeamos dos veces sobre un contacto

    Realiza una llamada al numero asociado al contacto que tengamos seleccionado.
    */
    void llamar();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'mostrar'

    Muestra el listado de llamadas recibidas mientras este el programa funcionando.
    */
    void mostrarLlamadas();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'nuevo'

    Crea una nueva entrada con los parametro que le introduzcamos en el dialogo 'entrada'.
    */
    void nuevaEntrada();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'acerca de'

    Muestra un dialogo con informacion sobre la aplicacion (version, autores, etc...).
    */
    void sobre();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'guardar'

    Genera un fichero llamado 'telefonos.txt' con toda la informacion sobre nuestros contactos y algunas
    directivas de configuracion.
    */
    void guardar();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'borrar'

    Elimina el contacto que tengamos seleccionado en la tabla.
    */
    void borrar();

    /**
    @brief Slot que se ejecutara cuando seleccionemos la opcion del menu 'editar'

    Muestra un dialogo de edicion que nos permitira modificar la informacion del contacto
    que tengamos seleccionado.
    */
    void editar();

    /**
    @brief Slot que se ejecutara cuando activemos el icono de la barra de notificacion
    @param reason Razon por la cual se ha activado el icono.

    Realiza diferentes acciones en funcion de la razon por la cual se haya activado el icono como
    mostrar u ocultar el programa, mostrar el menu del icono del area de notificacion e ignorar
    ciertas acciones.
    */
    void iconoActivado(QSystemTrayIcon::ActivationReason reason);

    /**
    @brief Slot que se ejecutara cuando se produzcan cambios en la informacion de la tabla.
    @param topLeft Parametro sin usar pero necesario para poder conectar este slot con la seal dataChanged.
    @param bottomRight Parametro sin usar pero necesario para poder conectar este slot con la seal dataChanged.

    Nos permite saber si se han producido cambios en la informacion.
    */
    void datosEditados( const QModelIndex & topLeft, const QModelIndex & bottomRight );


public:

    /**
    @brief Constructor por defecto
    @param dirpath Contiene el path donde se encuentran las imagenes y los telefonos a cargar
    @param parent widget Padre
    */
    explicit Principal(QString dirpath = "./", QWidget *parent = 0);

    /**
    @brief Destructor por defecto

    Libera la memoria reservada
    */
    ~Principal();

private:

    /**
    @brief Crea un conjunto de acciones asociandolas a los slots anteriores

    Define un conjunto de acciones, asignandoles iconos, teclas de acceso rapido, informacion, etc....
    */
    void crearAcciones();

    /**
    @brief Crea la barra de menu

    Crea la barra de menus y los submenus correspondientes a la clasificacion de las acciones.
    */
    void crearMenus();

    /**
    @brief Crea el icono del area de notificacion

    Crea el icono del area de notificacion asignadole un icono y un menu.
    */
    void crearMenuNotificacion();

    /**
    @brief Crea el menu que se muestra cuando clickeamos con el boton derecho sobre un contacto

    Crea el menu que se muestra cuando clickeamos con el boton derecho sobre un contacto asignandole las
    acciones mas usuales que podriamos necesitar (editar, borrar, llamar, etc...).
    */
    void crearMenusContextuales();

    /**
    @brief Crea la barra de herramientas

    Crea una barra de herramientas que se mostrara debajo de la barra de menu mostrando las
    acciones mas usuales que podriamos necesitar (nuevo, editar, borrar, guardar, llamar, etc...).
    */
    void crearBarraHerramientas();

    /**
    @brief Aade una nueva entrada a la tabla de contactos
    @param nombre Contiene el nombre del contacto
    @param telefono Contiene el telefono del contacto
    @param imp Especifica si el contacto es importante o no
    */
    void addEntrada(const QString &nombre,const QString &telefono, const bool &imp = false);

    /**
    @brief Crea la tabla central con la informacion de los contactos


    Lee toda la informacion del fichero 'telefonos.txt' y carga tanto la configuracion del programa como toda
    la informacion de nuestros contactos. Muestra en el dialogo 'errores' los errores producidos al leer el fichero.
    */
    void crearAgenda();

    /**
    @brief Verifica que la cadena linea tenga el formato adecuado
    @param linea Cadena de entrada a verificar
    @param[out] err Contiene los errores encontrados en la cadena
    @param[out] nomb Contiene el trozo de la cadena correspndiente al nombre
    @param[out] tel Contiene el trozo de la cadena correspndiente al telefono
    @param[out] imp Contiene el trozo de la cadena correspndiente a si es importante o no
    @return true si el formato es valido o false si no
    */
    bool verificarCadena(QString linea, QString &err, QString &nomb, QString &tel, bool &imp);

    /**
    @brief Devuelve el numero de la fila seleccionada en la tabla
    @return numero de fila seleccionada
    */
    int filaSeleccionada();

    /**
    @brief Muestra un mensaje en el area de notificacion
    @param texto Mensaje a mostrar
    @param duracion Duracion del mensaje en segundos
    */
    void mostrarMensaje(QString texto, int duracion);

    /**
    @brief Muestra un dialogo de advertencia de si se quiere continuar o no
    @param titulo Titulo del mensaje
    @param texto Mensaje a mostrar
    @return true si se quiere continuar o false si no
    */
    bool okToContinue(QString titulo, QString texto);

    /**
    @brief Crea un socket udp y lo enlaza al pueto especificado por donde recibiremos la informacion de la llamada
    @param puerto Puerto por el cual se recibira la informacion
    */
    void crearServidor(int puerto);

    Ui::Principal *ui;
    Registro_llamadas * reg_llam;
    Llamada_saliente * llam_sal;
    Edicion * edi;
    Errores * erro;
    Entrada * ent;
    QUdpSocket *udpSocket; // Socket
    QSystemTrayIcon *trayIcon; // area de notificacion
    QMenu *helpMenu; // Menu ayuda
    QMenu *trayIconMenu; // Menu icono area de notificacion
    QMenu *fileMenu; // Menu archivo
    QMenu *editMenu; // Menu editar
    QMenu *viewMenu; // menu ver
    QToolBar *toolBar;
    QIcon icon; // Icono de la aplicacion
    Imagen image1, image2; // Imagenes
    bool mostrar_todos; // Indica si se van a filtrar las llamadas entrantes o no
    bool as_editado; // Indica si has editado algo o no, desde la ultima vez que guardaste
    QString cadena_mostrar;
    QAbstractItemModel *modelo; // Modelo tabla central
    QUrl url; // Url para llamar
    QString marca_imp; // Importante o no
    QString path; // Donde esta la aplicacion
    QNetworkAccessManager qnam; // Nos va a permitir realizar la llamada
    QAction *separatorAction; // Separador
    QAction *quitAction; // Accion de salir
    QAction *newAction; // Nuevo
    QAction *callAction; // llamar
    QAction *callShowAction; // mostrar llamadas
    QAction *saveAction; // guardar
    QAction *editAction; // editar
    QAction *aboutAction; // sobre aplicacion
    QAction *deleteAction; // Eliminar registro


};

#endif // PRINCIPAL_H

